﻿using System.Collections.Generic;
using System.Linq;
using FluentValidation;

namespace FluentValidationHarness
{
    public class PersonCreateRequestValidator : AbstractValidator<PersonCreateRequest>
    {
        public PersonCreateRequestValidator()
        {
            RuleFor(r => r.Firstname).NotEmpty();
            RuleFor(r => r.Lastname).NotEmpty();

            RuleFor(r => r.Addresses).NotNull();
            RuleFor(r => r.Addresses).NotEmpty();

            RuleFor(r => r.Addresses).Must(HaveAnActiveAndPrimary).WithMessage("One (and only one) address must be primary and active");
            RuleFor(r => r.Phones).Must(HaveAnActiveAndPrimary).WithMessage("One (and only one) phone must be primary and active");
            // this will not work with the above two rules.
            //RuleFor(r => r).Must(AtLeastOneActiveAndPrimaryBetweenPhoneAndAddress).WithName("request");
        }

        private bool AtLeastOneActiveAndPrimaryBetweenPhoneAndAddress(PersonCreateRequest p)
        {
            bool addressActiveAndPrimary = HaveAnActiveAndPrimary(p.Addresses);
            bool phoneActiveAndPrimary = HaveAnActiveAndPrimary(p.Phones);

            bool result = addressActiveAndPrimary || phoneActiveAndPrimary;

            return result;
        }

        private bool HaveAnActiveAndPrimary(IEnumerable<IActivePrimary> items)
        {
            if (items == null)
            {
                return false;
            }

            int activePrimary = items.Count(p => p.IsActive && p.IsPrimary);
            return (activePrimary == 1);
        }
    }
}
